from typing import Union, get_args, get_origin

primitive_types = {int, str, float, bool, type(None)}


def get_all_subclasses(cls):
    subclasses = cls.__subclasses__()
    return subclasses + [g for s in subclasses for g in get_all_subclasses(s)]


def is_primitive_type(field_type: type) -> bool:
    # Check if the type is a primitive type
    return field_type in primitive_types


def is_generic_composed_of_primitives(field_type: type) -> bool:
    origin = get_origin(field_type)
    if origin in {list, dict, tuple, Union}:
        # Check all arguments of the generic type
        args = get_args(field_type)
        return all(is_composed_of_primitives(arg) for arg in args)
    return False


def is_composed_of_primitives(field_type: type) -> bool:
    # Check if the type is a primitive type
    if is_primitive_type(field_type):
        return True

    # Check if the type is a generic type composed of primitives
    if is_generic_composed_of_primitives(field_type):
        return True

    return False


def to_snake_case(name: str) -> str:
    # Examples:
    # "HelloWorld" -> "hello_world"
    # "helloWorld" -> "hello_world"
    # "hello_world" -> "hello_world"
    # "HTTPResponse" -> "http_response"
    # "REQUEST" -> "request"

    result = []
    for i, char in enumerate(name):
        if char.isupper():
            # If it's the first character, just add lowercase
            if i == 0:
                result.append(char.lower())
            else:
                # If previous char was lowercase, add underscore
                if name[i - 1].islower():
                    result.append("_" + char.lower())
                # If previous char was uppercase, just add lowercase
                else:
                    result.append(char.lower())
        else:
            result.append(char)

    return "".join(result)


def is_optional(field_type: type) -> bool:
    return get_origin(field_type) is Union and type(None) in get_args(field_type)


def is_list(field_type: type) -> bool:
    # Check if the field type is a List
    return get_origin(field_type) is list


def is_dict(field_type: type) -> bool:
    # Check if the field type is a Dict
    return get_origin(field_type) is dict


def get_inner_type(field_type: type) -> type:
    return next(t for t in get_args(field_type) if t is not type(None))


def is_subclass(cls, parent: type) -> bool:
    return hasattr(cls, "__bases__") and parent in cls.__bases__
